/***************************************************************************

  Arcade machine emulator V0.01 04/27/98


  Z80 engine by Marcel de Kogel
  Emulator by Jim Skilskyj based on the structure
  of the emulators by Brian Peek and Chris Hardy.
  (read the emulators howto by them to learn more)
  Also some code ideas from the Phoenix emulator by Richard Davies

  This emulator is only to guide you on how to use the Z80 emulator 
  and how to write code to emulate an arcade machine.

***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <conio.h>
#include <keys.h>
#include "z80.h"

// Change 8000 (32K RAM) to whatever is needed
// Also look in Z80.h for IPeriod = 12000 (set for Phoenix currently)

unsigned char RAM[0x8000];
FILE *in;
Z80_Regs regs;

int Z80_IRQ=-1;
int Z80_Iperiod = 120000;

int DipSwitchSYNC = 0;
int main(int argc,char **argv)
{
		
clrscr();
textmode(C4350);

printf("\nGeneric Arcade Machine Emulator v0.01 4/27/98\n"
"The following are standard keyboard definitions.\n"
"\n\n"
" Game keys   3       Insert coin\n"
"             1       Start 1 player game\n"
"             2       Start 2 player game\n"
"             Arrows  Move Around\n"
"             P       Pause\n"
"             ESC     Exit emulator\n"
"\n\n"
"Press <ENTER> to continue.\n");

getch();
clrscr();

if (Load_ROMs() == 0)
	{
	if (run_machine() != 0)
		{
		printf("Unable to start emulation\n");
		machine_exit();
		}
	else printf("Unable to initialize system\n");
	machine_exit();
	}
exit(0);
}

int run_machine()
{
Z80_Reset();
Z80();  /*  start the CPU emulation */
return (0);
}

int Load_ROMs()
{
// This will load the ROMs 

in = fopen("ic45", "rb");
if (!in)
	{
	printf("Cannot load IC45.\n");
	exit (0);
	}
fread(&RAM[0x0000], 0x0800, 1, in);
fclose(in);

in = fopen("ic46", "rb");
if (!in)
	{
	printf("Cannot load IC46.\n");
	exit (0);
	}
fread(&RAM[0x0800], 0x0800, 1, in);
fclose(in);

in = fopen("ic47", "rb");
if (!in)
	{
	printf("Cannot load IC47.\n");
	exit (0);
	}
fread(&RAM[0x1000], 0x0800, 1, in);
fclose(in);

in = fopen("ic48", "rb");
if (!in)
	{
	printf("Cannot load IC48.\n");
	exit (0);
	}
fread(&RAM[0x1800], 0x0800, 1, in);
fclose(in);

in = fopen("ic49", "rb");
if (!in)
	{
	printf("Cannot load IC49.\n");
	exit (0);
	}
fread(&RAM[0x2000], 0x0800, 1, in);
fclose(in);

in = fopen("ic50", "rb");
if (!in)
	{
	printf("Cannot load IC50.\n");
	exit (0);
	}
fread(&RAM[0x2800], 0x0800, 1, in);
fclose(in);

in = fopen("ic51", "rb");
if (!in)
	{
	printf("Cannot load IC51.\n");
	exit (0);
	}
fread(&RAM[0x3000], 0x0800, 1, in);
fclose(in);

in = fopen("ic52", "rb");
if (!in)
	{
	printf("Cannot load IC52.\n");
	exit (0);
	}
fread(&RAM[0x3800], 0x0800, 1, in);
fclose(in);

// For debug purposes
printf("ROMs Loaded\n");

return(0);
}

int machine_exit()
{
printf("This will exit the machine\n");
exit(0);
}


// z80 kernel functions

unsigned Z80_RDMEM (dword A)
{
// gotoxy(1,34);
// printf("Executing location %04X\n", A);
 

if(A >= 0x7800 && A <=0x7bff)
	
	{
	if(DipSwitchSYNC == 1)
		{
		VideoDraw();
		DipSwitchSYNC = 0;
		return 128;
		}
		else
		return 0;
	}
	else if (A < 0x8000)
	return RAM[A];
	// Warn you if you read RAM greater than 8000
	printf("WARNING! Reading from location %04X\n", A);
	return RAM[A];

}

void Z80_WRMEM(dword A,byte V)
{
if (A < 0x3000)
	{
	// Attempting to write to ROM, just return");
	return;
	}
	else if(A > 0x3fff && A < 0x8000)
//        gotoxy (1,33);
//        printf("Writing to location %04X\n", A);
	RAM[A] = V;
return;
}

/*
byte DoIn(byte A)
{
printf("Blank DoIn.\n");
return(0);
}

void DoOut(byte A,byte V)
{
printf("Blank DoOut.\n");
return;
}
*/

int InterruptsEnabled(void)
{
return;
}

byte Z80_In(byte Port)
{
return(0);
}
void Z80_Out(byte Port,byte Value)
{
return;
}
void Z80_Reti(void)
{
return;
}
void Z80_Retn(void)
{
return;
}

int Z80_Interrupt(void)
{
DipSwitchSYNC = 1;
return(Z80_IGNORE_INT);
}

void Z80_Patch(Z80_Regs *Regs)
{ 
printf("Blank Patch.\n");
return;
}

void Debug()
{
printf("Blank Debugger.\n");
return;
}

int VideoDraw(dword A)
{
int x,y;
unsigned char c;

	gotoxy(1,1);
	for(y = 0; y < 32; y++)
		{
		int pos = 0x4000+32*(26-1)+y;
		for (x = 0; x < 26; x++)
			{
			c = RAM[pos];
			if(c>=1 && c<=27)
			c+='A'-1;
			else if (c==31)
			c='*';
			else if (c > 30 && c < 42)
			c+= 16;
			else if (c == 43)
			c='-';
			else if (c == 42)
			c='.';
			else if (c > 0x00)
			c='X';
			printf("%c",c);
			pos -= 32;
			}
		printf("\n");
		
		}

		// Now draw background chars (doesn't scroll yet)
	gotoxy(1,1);
	for(y = 0; y < 32; y++)
		{
		int pos = 0x4800+32*(26-1)+y;
		for (x = 0; x < 26; x++)
			{
			c = RAM[pos];
			if(c>=1 && c<=27)
			c='.';
			else if (c==31)
			c='*';
			else if (c > 30 && c < 42)
			c= '.';
			else if (c == 43)
			c='-';
			else if (c == 42)
			c='.';
			else if (c > 0x00)
			c='O';
			printf("%c",c);
			pos -= 32;
			}
		printf("\n");
		
		}


return(0);

}


/*
   
   Future keyboard routine


int Keyboard_in(res)
{
   Read DOS keyboard routine

}

*/
